import logging

from contracts_lib_py.templates.template_base import TemplateBase

logger = logging.getLogger('nftSalesTemplate')


class NFTSalesTemplate(TemplateBase):
    """Class representing the NFTSalesTemplate contract."""
    CONTRACT_NAME = 'NFTSalesTemplate'
