import logging

from contracts_lib_py.templates.template_base import TemplateBase

logger = logging.getLogger('didSalesTemplate')


class DIDSalesTemplate(TemplateBase):
    """Class representing the DIDSalesTemplate contract."""
    CONTRACT_NAME = 'DIDSalesTemplate'
